@@ -4,6 +4,7 @@
 
 """Class for reading and writing uvfits files."""
 import os
+import copy
 import warnings
 
 import numpy as np
@@ -133,6 +134,40 @@ class UVFITS(UVData):
         # initialize internal variables based on the antenna lists
         self.Nants_data = int(np.union1d(self.ant_1_array, self.ant_2_array).size)
 
+        # check for suffixes in the baseline coordinate names indicating the
+        # baseline coordinate system
+        if (
+            "UU" in vis_hdu.data.parnames
+            and "VV" in vis_hdu.data.parnames
+            and "WW" in vis_hdu.data.parnames
+        ):
+            uvw_names = ["UU", "VV", "WW"]
+        elif (
+            "UU---SIN" in vis_hdu.data.parnames
+            and "VV---SIN" in vis_hdu.data.parnames
+            and "WW---SIN" in vis_hdu.data.parnames
+        ):
+            uvw_names = ["UU---SIN", "VV---SIN", "WW---SIN"]
+        elif (
+            "UU---NCP" in vis_hdu.data.parnames
+            and "VV---NCP" in vis_hdu.data.parnames
+            and "WW---NCP" in vis_hdu.data.parnames
+        ):
+            uvw_names = ["UU---NCP", "VV---NCP", "WW---NCP"]
+            warnings.warn(
+                "The baseline coordinates (uvws) in this file are specified in the "
+                "---NCP coordinate system, which is does not agree with our baseline "
+                "coordinate conventions. Rotating the uvws to match our convention "
+                "(Note that this rotation has not been widely tested)."
+            )
+        else:
+            raise ValueError(
+                "There is no consistent set of baseline coordinates in this file. "
+                "The UU, VV and WW coordinate must have no suffix or the '---SIN' or "
+                "'---NCP' suffix and the suffixes must match on all three baseline "
+                "coordinate parameters."
+            )
+
         # read baseline vectors in units of seconds, return in meters
         # FITS uvw direction convention is opposite ours and Miriad's.
         # So conjugate the visibilities and flip the uvws:
@@ -140,9 +175,9 @@ class UVFITS(UVData):
             np.array(
                 np.stack(
                     (
-                        vis_hdu.data.par("UU"),
-                        vis_hdu.data.par("VV"),
-                        vis_hdu.data.par("WW"),
+                        vis_hdu.data.par(uvw_names[0]),
+                        vis_hdu.data.par(uvw_names[1]),
+                        vis_hdu.data.par(uvw_names[2]),
                     )
                 )
             )
@@ -161,8 +196,13 @@ class UVFITS(UVData):
                     np.ones_like(self.time_array, dtype=np.float64) * int_time
                 )
             else:
-                raise ValueError(
-                    "integration time not specified and only one time present"
+                warnings.warn(
+                    "The integration time is not specified and only one time is "
+                    "present so it cannot be calculated from the difference between "
+                    "integration times. Setting to None which will cause the check to "
+                    "error. Set `run_check` to False to read in the file without "
+                    "checking. Then set the integration_time (to an array of length "
+                    "Nblts) directly on the object to allow futher processing."
                 )
 
         if proc is not None:
@@ -324,6 +364,7 @@ class UVFITS(UVData):
                 check_extra=check_extra,
                 run_check_acceptability=run_check_acceptability,
                 strict_uvw_antpos_check=strict_uvw_antpos_check,
+                allow_flip_conj=True,
             )
 
     def read_uvfits(
@@ -560,7 +601,12 @@ class UVFITS(UVData):
             ):
                 self.history += self.pyuvdata_version_str
 
-            self.vis_units = vis_hdr.pop("BUNIT", "UNCALIB")
+            self.vis_units = vis_hdr.pop("BUNIT", "uncalib")
+            # Added here as a fix since some previous versions of UVData allowed for
+            # all caps versions of UNCALIB.
+            if self.vis_units == "UNCALIB":
+                self.vis_units = "uncalib"
+
             self.phase_center_epoch = vis_hdr.pop("EPOCH", None)
 
             # PHSFRAME is not a standard UVFITS keyword, but was used by older
@@ -612,23 +658,11 @@ class UVFITS(UVData):
             if "FRAME" in ant_hdu.header.keys():
                 xyz_telescope_frame = ant_hdu.header["FRAME"]
             else:
-                if "COMMENT" in self.extra_keywords:
-                    if uvutils._check_history_version(
-                        self.extra_keywords["COMMENT"],
-                        "Created by Cotter MWA preprocessor",
-                    ):
-                        # this is a Cotter file, which should have the frame set to ITRF
-                        warnings.warn(
-                            "Required Antenna frame keyword not set, but this appears "
-                            "to be a Cotter file, setting to ITRF."
-                        )
-                        xyz_telescope_frame = "ITRF"
-                else:
-                    warnings.warn(
-                        "Required Antenna frame keyword not set, setting to ????"
-                    )
-                    xyz_telescope_frame = "????"
-
+                warnings.warn(
+                    "Required Antenna keyword 'FRAME' not set; "
+                    "Assuming frame is 'ITRF'."
+                )
+                xyz_telescope_frame = "ITRF"
             # get telescope location and antenna positions.
             # VLA incorrectly sets ARRAYX/ARRAYY/ARRAYZ to 0, and puts array center
             # in the antenna positions themselves
@@ -784,6 +818,13 @@ class UVFITS(UVData):
             # Calculate the apparent coordinate values
             self._set_app_coords_helper()
 
+            # fix up the uvws if in the NCP baseline coordinate frame.
+            # Must be done here because it requires the phase_center_app_dec
+            if "UU---NCP" in vis_hdu.data.parnames:
+                self.uvw_array = uvutils._rotate_one_axis(
+                    self.uvw_array[:, :, None], self.phase_center_app_dec - np.pi / 2, 0
+                )[:, :, 0]
+
             if not read_data:
                 # don't read in the data. This means the object is a metadata
                 # only object but that may not matter for many purposes.
@@ -828,6 +869,12 @@ class UVFITS(UVData):
         """
         Write the data to a uvfits file.
 
+        If using this method to write out a data set for import into CASA, users should
+        be aware that the `importuvifts` task does not currently support reading in
+        data sets where the number of antennas is > 255. If writing out such a data set
+        for use in CASA, we suggest using the measurement set writer (`UVData.write_ms`)
+        instead.
+
         Parameters
         ----------
         filename : str
@@ -1026,12 +1073,15 @@ class UVFITS(UVData):
         # FITS uvw direction convention is opposite ours and Miriad's.
         # So conjugate the visibilities and flip the uvws:
         uvw_array_sec = -1 * self.uvw_array / const.c.to("m/s").value
-        # jd_midnight = np.floor(self.time_array[0] - 0.5) + 0.5
-        tzero = np.float32(self.time_array[0])
 
-        # uvfits convention is that time_array + relevant PZERO = actual JD
-        # We are setting PZERO4 = float32(first time of observation)
-        time_array = np.float32(self.time_array - np.float64(tzero))
+        # uvfits convention is that there are two float32 time_arrays and the
+        # float64 sum of them + relevant PZERO = actual JD
+        # a common practice is to set the PZERO to the JD at midnight of the first time
+        jd_midnight = np.floor(self.time_array[0] - 0.5) + 0.5
+        time_array1 = np.float32(self.time_array - jd_midnight)
+        time_array2 = np.float32(
+            self.time_array - jd_midnight - np.float64(time_array1)
+        )
 
         int_time_array = self.integration_time
 
@@ -1041,11 +1091,13 @@ class UVFITS(UVData):
         # Set up dictionaries for populating hdu
         # Note that uvfits antenna arrays are 1-indexed so we add 1
         # to our 0-indexed arrays
+
         group_parameter_dict = {
             "UU      ": uvw_array_sec[:, 0],
             "VV      ": uvw_array_sec[:, 1],
             "WW      ": uvw_array_sec[:, 2],
-            "DATE    ": time_array,
+            "DATE    ": time_array1,
+            "DATE2   ": time_array2,
             "BASELINE": baselines_use,
             "SOURCE  ": None,
             "FREQSEL ": np.ones_like(self.time_array, dtype=np.float32),
@@ -1069,6 +1121,7 @@ class UVFITS(UVData):
             "VV      ": 1.0,
             "WW      ": 1.0,
             "DATE    ": 1.0,
+            "DATE2   ": 1.0,
             "BASELINE": 1.0,
             "SOURCE  ": 1.0,
             "FREQSEL ": 1.0,
@@ -1081,7 +1134,8 @@ class UVFITS(UVData):
             "UU      ": 0.0,
             "VV      ": 0.0,
             "WW      ": 0.0,
-            "DATE    ": tzero,
+            "DATE    ": jd_midnight,
+            "DATE2   ": 0.0,
             "BASELINE": 0.0,
             "SOURCE  ": 0.0,
             "FREQSEL ": 0.0,
@@ -1105,12 +1159,19 @@ class UVFITS(UVData):
 
         # list contains arrays of [u,v,w,date,baseline];
         # each array has shape (Nblts)
-        parnames_use = ["UU      ", "VV      ", "WW      ", "DATE    "]
+        parnames_use = ["UU      ", "VV      ", "WW      ", "DATE    ", "DATE2   "]
         if np.max(self.ant_1_array) < 255 and np.max(self.ant_2_array) < 255:
             # if the number of antennas is less than 256 then include both the
             # baseline array and the antenna arrays in the group parameters.
             # Otherwise just use the antenna arrays
             parnames_use.append("BASELINE")
+        else:
+            warnings.warn(
+                "Found antenna numbers > 256 in this data set. This is permitted by "
+                "UVFITS standards, but may cause the `importuvfits` utility within "
+                "CASA to crash. If attempting to use this data set in CASA, consider "
+                "using the measurement set writer method (`write_ms`) instead."
+            )
 
         if self.multi_phase_center:
             parnames_use.append("SOURCE  ")
@@ -1129,9 +1190,11 @@ class UVFITS(UVData):
             parnames_use.append("LST     ")
             group_parameter_list.append(lst_array_2)
 
+        parnames_use_datefix = copy.deepcopy(parnames_use)
+        parnames_use_datefix[parnames_use_datefix.index("DATE2   ")] = "DATE    "
         hdu = fits.GroupData(
             uvfits_array_data,
-            parnames=parnames_use,
+            parnames=parnames_use_datefix,
             pardata=group_parameter_list,
             bitpix=-32,
         )
@@ -1423,10 +1486,10 @@ class UVFITS(UVData):
                         self.phase_center_app_dec[self.phase_center_id_array == cat_id]
                     )
 
-                ra_arr *= 180.0 / np.pi
-                dec_arr *= 180.0 / np.pi
-                app_ra *= 180.0 / np.pi
-                app_dec *= 180.0 / np.pi
+            ra_arr *= 180.0 / np.pi
+            dec_arr *= 180.0 / np.pi
+            app_ra *= 180.0 / np.pi
+            app_dec *= 180.0 / np.pi
 
             col_list = [
                 fits.Column(name="ID. NO.", format="1J", array=sou_ids),
